ALLOWED_HOSTS = ["localhost", "127.0.0.1"]
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ".tox/test.sqlite3",
    }
}
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
DEBUG = True
INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "rosetta",
    "django_bootstrap5",
    "testproject",
    "siruta",
]
LANGUAGES = [
    ("en", "EN"),
    ("ro", "RO"),
]
LOCALE_PATHS = ["src/siruta/locale"]
LOGIN_REDIRECT_URL = "/"
LOGIN_URL = "/admin/login/"
MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
]
PHONENUMBER_DB_FORMAT = "NATIONAL"
PHONENUMBER_DEFAULT_REGION = "RO"
PRODUCER_SLEEP_SECONDS = None
REST_FRAMEWORK = {}
ROOT_URLCONF = "testproject.urls"
SECRET_KEY = __name__
STATIC_URL = "/static/"
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "debug": DEBUG,
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    }
]
TIME_ZONE = "Europe/Bucharest"
USE_I18N = True
USE_TZ = True
API_TOKEN = None



extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.coverage",
    "sphinx.ext.doctest",
    "sphinx.ext.extlinks",
    "sphinx.ext.ifconfig",
    "sphinx.ext.napoleon",
    "sphinx.ext.todo",
    "sphinx.ext.viewcode",
]
source_suffix = ".rst"
master_doc = "index"
project = "django-siruta"
year = "2025"
author = "Ionel Cristian Mărieș"
copyright = f"{year}, {author}"
version = release = "0.2.1"

pygments_style = "trac"
templates_path = ["."]
extlinks = {
    "issue": ("https://github.com/ionelmc/django-siruta/issues/%s", "#%s"),
    "pr": ("https://github.com/ionelmc/django-siruta/pull/%s", "PR #%s"),
}

html_theme = "furo"
html_theme_options = {
    "source_repository": "https://github.com/ionelmc/django-siruta/",
    "source_branch": "main",
    "source_directory": "docs/",
    "footer_icons": [
        {
            "url": "https://github.com/ionelmc/django-siruta/",
            "html": "github.com/ionelmc/django-siruta",
        },
    ],
}

html_use_smartypants = True
html_last_updated_fmt = "%b %d, %Y"
html_split_index = False
html_short_title = f"{project}-{version}"

napoleon_use_ivar = True
napoleon_use_rtype = False
napoleon_use_param = False



import json

from django import forms
from django.views import generic

from siruta.forms import CountyField
from siruta.forms import LocalityField


class DemoForm(forms.Form):
    county = CountyField(label="Delivery county")
    locality = LocalityField(label="Delivery locality")

    billing_county = CountyField()
    billing_locality = LocalityField(county_field="billing_county")


class DemoView(generic.FormView):
    template_name = "demo.html"
    form_class = DemoForm
    success_url = "/thanks/"

    def form_valid(self, form):
        return self.render_to_response(self.get_context_data(data=json.dumps(form.cleaned_data, indent=4)))



import json

from django import forms
from django.core.exceptions import ValidationError
from django.forms import widgets
from django.utils.choices import normalize_choices
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _

from .data import COUNTIES_BY_ID
from .extras import LOCALITIES_BY_COUNTY_ID
from .extras import LOCALITIES_BY_ID

COUNTY_CHOICES = (("", ""), *COUNTIES_BY_ID.items())
LOCALITY_JSON = json.dumps(
    {
        county_id: [{"text": locality_name, "value": locality_id} for locality_id, locality_name in localities.items()]
        for county_id, localities in LOCALITIES_BY_COUNTY_ID.items()
    }
)


class SelectizeSelectWidget(widgets.Select):
    template_name = "siruta/selectize_select.html"


class CountyField(forms.TypedChoiceField):
    """
    An integer field with choices.
    """

    def __init__(self, **kwargs):
        super().__init__(choices=COUNTY_CHOICES, coerce=int, **kwargs)


class LocalityWidget(widgets.TextInput):
    template_name = "siruta/locality.html"

    def __init__(self, *, county_field):
        super().__init__()
        self.county_field = county_field

    def get_context(self, name, value, attrs):
        attrs = {
            **attrs,
            "disabled": "disabled",
        }
        return {
            "county_field": self.county_field,
            "localities": mark_safe(LOCALITY_JSON),
            **super().get_context(name, value, attrs),
        }

    def value_from_datadict(self, data, files, name):
        value = super().value_from_datadict(data, files, name)
        return value, data.get(self.county_field)

    def format_value(self, value):
        if isinstance(value, tuple):
            value, _ = value
        return super().format_value(value)


class LocalityField(forms.Field):
    """
    An integer field with choices.

    The default widget dynamically changes choices depending on another form field named ``"county"``.
    Override ``county_field`` to change this.

    Values are ordered by type.
    See ``siruta.cli.SIRUTA_TYPE_REORDER`` for exact precedence.
    """

    def __init__(self, *, county_field="county", **kwargs):
        super().__init__(
            widget=LocalityWidget(county_field=county_field),
            **kwargs,
        )
        self.choices = normalize_choices(LOCALITIES_BY_ID)

    def to_python(self, value):
        locality_id, county_id = value

        if locality_id:
            try:
                locality_id = int(locality_id)
                county_id = int(county_id)
            except ValueError as exc:
                raise ValidationError(_("Invalid value {value}.").format(value=repr(value))) from exc

            county = COUNTIES_BY_ID.get(county_id)
            if not county:
                raise ValidationError(_("Invalid county ID '{id}'.").format(id=county_id))

            locality = LOCALITIES_BY_ID.get(locality_id)
            if not locality:
                raise ValidationError(_("Invalid locality ID '{id}'.").format(id=locality_id))

            county_localities = LOCALITIES_BY_COUNTY_ID[county_id]
            if locality_id not in county_localities:
                raise ValidationError(
                    _("Locality '{locality}' is not valid for county '{county}'.").format(
                        county=county,
                        locality=locality,
                    )
                )

            return locality_id
        else:
            return None



import re

from unidecode import unidecode

from .data import LOCALITIES_BY_COUNTY_ID


def clean_locality_name(value, regex=re.compile(r"[-.\s]+")):
    return regex.sub(" ", unidecode(value)).strip().upper()


LOCALITIES_BY_ID = {
    locality_id: locality_name for localities in LOCALITIES_BY_COUNTY_ID.values() for locality_id, locality_name in localities.items()
}
LOCALITIES_BY_COUNTY_ID_BY_NAME = {
    county_id: {clean_locality_name(locality_name): locality_id for locality_id, locality_name in localities.items()}
    for county_id, localities in LOCALITIES_BY_COUNTY_ID.items()
}



"""
Module that contains the command line app.

Why does this file exist, and why not put this in __main__?

  You might be tempted to import things from __main__ later, but that will cause
  problems: the code will get executed twice:

  - When you run `python -msiruta` python will execute
    ``__main__.py`` as a script. That means there will not be any
    ``siruta.__main__`` in ``sys.modules``.
  - When you import __main__ it will get executed again (as a module) because
    there"s no ``siruta.__main__`` in ``sys.modules``.

  Also see (1) from http://click.pocoo.org/5/setuptools/#setuptools-integration
"""

from __future__ import annotations

import argparse
import csv
import enum
import json
import operator
from pathlib import Path

import msgspec
from unidecode import unidecode

data_path = Path("data")
parser = argparse.ArgumentParser(description="Command description.")
parser.add_argument(
    dest="localities_file",
    metavar="CSV",
    default=data_path / "siruta_an_2024.csv",
    nargs="?",
    type=Path,
    help="Input CSV file. Default: %(default)s",
)
parser.add_argument(
    dest="counties_file",
    metavar="CSV",
    default=data_path / "siruta-judete.csv",
    nargs="?",
    type=Path,
    help="Input CSV file. Default: %(default)s",
)
parser.add_argument(
    "--dialect",
    "-d",
    default="excel",
    help=f"CSV dialect to use. Available: {csv.list_dialects()}",
)
parser.add_argument(
    "--output-path",
    "-o",
    default=Path("src", "siruta"),
    type=Path,
)


class SirutaTypes(enum.IntEnum):
    """
    https://legislatie.just.ro/public/DetaliiDocument/46083::

        0	Judet, municipiul Bucuresti
        1	Municipiu
        2	Oras ce apartine de judet
        3	Comuna
        4	Oras declarat municipiu
        5	Oras ce apartine de municipiu
        6	Comuna suburbana ce apartine de municipiu
        7	Oras fara localitate subordonata, ce apartine de judet
        8	Comuna suburbana ce apartine de judet
        9	Localitate componenta a orasului de resedinta declarata municipiu
        10	Celelalte localitati sau sectoare componente ale orasului declarat municipiu
        11	Sat ce apartine de orasul declarat municipiu
        12	Localitate componenta (resedinta orasului ce apartine de municipiu)
        13	Celelalte localitati componente ce apartin orasului care este subordonat municipiului
        14	Sat apartinind unui oras subordonat municipiului
        15	Sat resedinta de comuna suburbana care apartine de municipiu
        16	Celelalte sate care apartin comunei suburbane subordonate unui municipiu
        17	Localitate componenta (resedinta orasului care apartine de judet)
        18	Celelalte localitati componente ale uni oras ce apartine de judet
        19	Sate subordonate unui oras ce apartine de judet
        20	Sat resedinta de comuna suburbana subordonata unui oras care apartine de judet
        21	Celelalte sate apartinind de o comuna suburbana subordonate unui oras apartinind judetului
        22	Sat resedinta de comuna
        23	Sat component al comunei
    """

    judet = 40  # Judet, municipiul Bucuresti
    municipiu_rdj = 1  # Municipiu resedinta de judet, municipiul Bucuresti
    oras = 2  # Oras ce apartine de judet, altele decit  resedinta de judet
    comuna = 3  # Comuna
    municipiu = 4  # Municipiu, altele decit resedinta de judet
    oras_rdj = 5  # Oras resedinta de judet
    sector = 6  # Sectoarele municipiului Bucuresti
    localitate_crm = 9  # Localitate componenta, resedinta de municipiu
    localitate_cm = 10  # Localitate componentă, a unui municipiu alta decât reşedinţă de municipiu
    sat_m = 11  # Sat ce apartine de municipiu
    localitate_cro = 17  # Localitate componenta (resedinta orasului care apartine de judet)
    localitate_co = 18  # Localitati componente ale orasului altele decit resedinta de oras
    sat_o = 19  # Sate subordonate unui oras
    sat_rc = 22  # Sat resedinta de comuna
    sat_c = 23  # Sat ce apartine de comuna, altele decit resedinta de comuna


SIRUTA_TYPE_REORDER = (
    SirutaTypes.judet,
    SirutaTypes.sector,
    SirutaTypes.municipiu_rdj,
    SirutaTypes.municipiu,
    SirutaTypes.localitate_crm,
    SirutaTypes.oras_rdj,
    SirutaTypes.oras,
    SirutaTypes.localitate_cro,
    SirutaTypes.localitate_cm,
    SirutaTypes.localitate_co,
    SirutaTypes.sat_m,
    SirutaTypes.sat_o,
    SirutaTypes.comuna,
    SirutaTypes.sat_rc,
    SirutaTypes.sat_c,
)


class SirutaLevel(enum.IntEnum):
    judet = 1  # Judeţe, Municipiul Bucureşti
    municipiu = 2  # Municipii
    localitate = 5  # Localităţi componente
    oras = 3  # Oraşe
    sat = 6  # Sate
    comuna = 4  # Comune
    sector = 7  # Sectoare ale capitalei


class SirutaArea(enum.IntEnum):
    judet = 0
    urban = 1
    rural = 3


class SirutaCountyRow(msgspec.Struct):
    id: int = msgspec.field(name="JUD")
    name: str = msgspec.field(name="DENJ")
    order: int = msgspec.field(name="FSJ")
    code: str = msgspec.field(name="MNEMONIC")
    area: int = msgspec.field(name="ZONA")


class SirutaRow(msgspec.Struct):
    id: int = msgspec.field(name="SIRUTA")
    name: str = msgspec.field(name="DENLOC")
    county: int = msgspec.field(name="JUD")
    parent: int = msgspec.field(name="SIRSUP")
    type: SirutaTypes = msgspec.field(name="TIP")
    level: SirutaLevel = msgspec.field(name="NIV")
    area: SirutaArea = msgspec.field(name="MED")
    region: int = msgspec.field(name="REGIUNE")
    county_order: int = msgspec.field(name="FSJ")
    alpha_order: str = msgspec.field(name="FSL")

    ascii_name: str = None
    children: list = msgspec.field(default_factory=list)

    def __post_init__(self):
        self.ascii_name = unidecode(self.name)


SIRUTA_ROW_ORDERING_KEY = operator.attrgetter("ordering_key")


def run(args=None):
    args = parser.parse_args(args=args)
    county_codes = {}
    with args.counties_file.open() as stream:
        reader = csv.DictReader(stream, dialect=args.dialect, delimiter=";")
        fields = reader.fieldnames
        print(f"Reading {args.counties_file!r} with fields: {fields!r}")
        for row in reader:
            row = msgspec.convert(row, type=SirutaCountyRow, strict=False)
            county_codes[row.id] = row.code

    with args.localities_file.open() as stream:
        reader = csv.DictReader(stream, dialect=args.dialect)
        fields = reader.fieldnames
        print(f"Reading {args.localities_file!r} with fields: {fields!r}")
        counties = {}
        localities_by_county = {}
        entries = {}
        rows = {}
        for row in reader:
            row = msgspec.convert(row, type=SirutaRow, strict=False)
            rows[row.id] = row

            if row.type == SirutaTypes.judet:
                counties[row.county] = row.name.removeprefix("JUDETUL ").removeprefix("JUDEŢUL ").removeprefix("MUNICIPIUL ")
                assert row.county in county_codes
                localities_by_county[row.county] = []

            entries[row.id] = row
            if row.parent > 1:
                entries[row.parent].children.append(row)

        for row in entries.values():
            if not (row.type == SirutaTypes.judet or row.children):
                localities_by_county[row.county].append(row)

        def ordering_key(row):
            if row.parent > 1:
                return SIRUTA_TYPE_REORDER.index(row.type), *ordering_key(rows[row.parent]), row.ascii_name
            else:
                return SIRUTA_TYPE_REORDER.index(row.type), row.ascii_name

        localities_by_county_by_id = {}
        for county, localities in localities_by_county.items():
            localities_by_county_by_id[county] = localities_by_id = {}
            localities.sort(key=ordering_key)
            for locality in localities:
                localities_by_id[locality.id] = locality.name

        with args.output_path.joinpath("data.py").open("w") as consts_fh:
            consts_fh.write("COUNTY_IDS_BY_CODE = {\n    ")
            consts_fh.write("\n    ".join(f'"{v}": {k!r},' for k, v in county_codes.items()))
            consts_fh.write("\n}\n")
            consts_fh.write("COUNTY_CODES_BY_ID = {code: county_id for county_id, code in COUNTY_IDS_BY_CODE.items()}\n\n")
            consts_fh.write("COUNTIES_BY_ID = {\n    ")
            consts_fh.write("\n    ".join(f'{k!r}: "{v}",' for k, v in counties.items()))
            consts_fh.write("\n}\n\nLOCALITIES_BY_COUNTY_ID = {")
            for county_id, localities in localities_by_county_by_id.items():
                consts_fh.write(f"\n    {county_id!r}: {{  # {counties[county_id]}\n        ")
                consts_fh.write("\n        ".join(f'{k!r}: "{v}",' for k, v in localities.items()))
                consts_fh.write("\n    },")
            consts_fh.write("\n}\n")

        with args.output_path.joinpath("static", "siruta", "counties.js").open("w") as fh:
            fh.write("const COUNTIES = ")
            json.dump(
                {
                    "RO": [
                        {"value": "", "text": ""},
                        *[{"value": county_codes[county_id], "text": name} for county_id, name in counties.items()],
                    ]
                },
                fh,
            )
            fh.write(";\nconst LOCALITIES = ")
            json.dump(
                {
                    county_codes[county_id]: [
                        {"value": "", "text": ""},
                        *[{"value": unidecode(name), "text": name} for name in localities.values()],
                    ]
                    for county_id, localities in localities_by_county_by_id.items()
                },
                fh,
            )
            fh.write(";\n")



import subprocess


def test_main():
    assert (
        subprocess.check_output(["import-siruta-csv"], text=True)
        == """Reading PosixPath('data/siruta-judete.csv') with fields: ['JUD', 'DENJ', 'FSJ', 'MNEMONIC', 'ZONA']
Reading PosixPath('data/siruta_an_2024.csv') with fields: ['SIRUTA', 'DENLOC', 'CODP', 'JUD', 'SIRSUP', 'TIP', 'NIV', 'MED', 'REGIUNE', 'FSJ', 'FSL', 'NUTS']
"""
    )



from django import forms

from siruta.forms import CountyField
from siruta.forms import LocalityField


class Form(forms.Form):
    locality = LocalityField()
    county = CountyField()


def test_form_valid():
    form = Form({"locality": "1026", "county": "1"})
    assert form.is_valid(), form.errors
    assert form.cleaned_data == {
        "county": 1,
        "locality": 1026,
    }


def test_form_bad_locality():
    form = Form({"locality": "1", "county": "1"})
    assert not form.is_valid()
    assert form.errors == {
        "locality": ["Invalid locality ID '1'."],
    }


def test_form_bad_locality_county():
    form = Form({"locality": "1", "county": "2"})
    assert not form.is_valid()
    assert form.errors == {
        "locality": ["Invalid locality ID '1'."],
    }


def test_form_bad_county():
    form = Form({"locality": "1", "county": "9999"})
    assert not form.is_valid()
    assert form.errors == {
        "county": ["Select a valid choice. 9999 is not one of the available choices."],
        "locality": ["Invalid county ID '9999'."],
    }



import pytest

from .views import DemoView

pytest.importorskip("django")

from django.contrib import admin
from django.urls import include
from django.urls import path

urlpatterns = [
    path("admin/", admin.site.urls),
    path("rosetta/", include("rosetta.urls")),
    path("demo", DemoView.as_view(), name="demo"),
]



#!/usr/bin/env python
import re
from pathlib import Path

from setuptools import find_namespace_packages
from setuptools import setup


def read(*names, **kwargs):
    with Path(__file__).parent.joinpath(*names).open(encoding=kwargs.get("encoding", "utf8")) as fh:
        return fh.read()


setup(
    long_description="{}\n{}".format(
        re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub("", read("README.rst")),
        re.sub(":[a-z]+:`~?(.*?)`", r"``\1``", read("CHANGELOG.rst")),
    ),
    long_description_content_type="text/x-rst",
    packages=find_namespace_packages("src"),
    package_dir={"": "src"},
    py_modules=[path.stem for path in Path("src").glob("*.py")],
    include_package_data=True,
    zip_safe=False,
)
